; Csound system for implementing reverbs based on nested all-pass filters
; similar to those used by William Gardner (MIT)
; Coded by Hans Mikelson 12/2/97
;
; 1. Noise Click
; 2. Disk Input
; 8. Simple Sum
; 9. Feedback Filter
;10. Delay
;11. Simple All-Pass Filter
;12. Nested All-Pass Filter
;13. Double Nested All-Pass Filter
;15. Output 

sr=44100
kr=22050
ksmps=2
nchnls=2
zakinit 30,30

;----------------------------------------------------------------------------------
; Noise Click for testing the decay curve of the reverb.
       instr  1

idur   =      p3
iamp   =      p4
ioutch =      p5
ifco   =      p6

kamp   linseg 0, .002, iamp, .002, 0, idur-.004, 0
aout   rand   kamp

afilt  butterlp aout, ifco
       zaw    afilt, ioutch
;       outs   aout, aout

       endin

;----------------------------------------------------------------------------------
; Disk Input
       instr  2

iamp   =      p4
irate  =      p5
isndin =      p6
ioutch =      p7
ifco   =      p8

ain    diskin  isndin, irate
; afilt  butterlp ain, ifco
;       zaw    afilt*iamp, ioutch
       zaw    ain*iamp, ioutch
       outs    ain*iamp, ain*iamp

       endin

;----------------------------------------------------------------------------------
; Band-Limited Impulse
       instr  3

iamp   =      p4
ifqc   =      cpspch(p5)
ioutch =      p6

apulse buzz    1,ifqc, sr/2/ifqc, 1 ; Avoid aliasing

       zaw     apulse*iamp, ioutch
       outs    apulse*iamp, apulse*iamp

       endin

       
;----------------------------------------------------------------------------------
; Simple Sum--Add 2 channels together and output to a third channel.
       instr  8

idur   =      p3
iinch1 =      p4
iinch2 =      p5
ioutch =      p6

ain1   zar    iinch1
ain2   zar    iinch2
       zaw    ain1+ain2, ioutch

       endin

;----------------------------------------------------------------------------------
; Feedback Filter
       instr  9

idur   =      p3
ifco   =      p4
igain  =      p5
iinch1 =      p6
iinch2 =      p7
ioutch =      p8

ain1   zar    iinch1
ain2   zar    iinch2                   ; Read in two channels

afilt  butterbp ain2, ifco, ifco/2     ; Bandpass filter one channel
       zaw    ain1+igain*afilt, ioutch ; Adjust filter level add & output

       endin

;----------------------------------------------------------------------------------
; Delay  -- Simple Delay
       instr  10

idur   =      p3
idtime =      p4/1000
igain  =      p5
iinch  =      p6
ioutch =      p7

ain    zar    iinch          ; Read the channel
aout   delay  ain, idtime    ; Delay for time
       zaw    aout, ioutch   ; Output the channel

       endin


;----------------------------------------------------------------------------------
; Simple All-Pass Filter
       instr  11

idur   =      p3
itime  =      p4/1000
igain  =      p5
iinch  =      p6
ioutch =      p7
adel1  init   0

ain    zar    iinch

aout   =      adel1-igain*ain           ; Feed Forward
adel1  delay  ain+igain*aout, itime     ; Delay and Feedback

       zaw    aout, ioutch

       endin

;----------------------------------------------------------------------------------
; Single Nested All-Pass Filter
       instr  12

idur   =      p3
itime1 =      p4/1000-p6/1000
igain1 =      p5
itime2 =      p6/1000
igain2 =      p7
iinch  =      p8
ioutch =      p9
adel1  init   0
adel2  init   0

ain    zar    iinch

asum   =      adel2 - igain2*adel1         ; Feed Forward (Inner all-pass)
aout   =      asum  - igain1*ain           ; Feed Forward (Outer all-pass)

adel1  delay  ain   + igain1*aout, itime1  ; Feedback (Outer all-pass)
adel2  delay  adel1 + igain2*asum, itime2  ; Feedback (Inner all-pass)

       zaw    aout, ioutch

       endin

;----------------------------------------------------------------------------------
; Double Nested All-Pass Filter (1 outer with 2 inner in series)
       instr  13

idur   =      p3
itime1 =      p4/1000-p6/1000-p8/1000
igain1 =      p5
itime2 =      p6/1000
igain2 =      p7
itime3 =      p8/1000
igain3 =      p9
iinch  =      p10
ioutch =      p11
adel1  init   0
adel2  init   0
adel3  init   0

ain    zar    iinch

asum1  =      adel2 - igain2*adel1         ; First  Inner Feed Forward
asum2  =      adel3 - igain3*asum1         ; Second Inner Feed Forward
aout   =      asum2 - igain1*ain           ; Outer  Feed Forward

adel1  delay  ain   + igain1*aout, itime1  ; Outer Feedback
adel2  delay  adel1 + igain2*asum1, itime2 ; First Inner Feedback
adel3  delay  asum1 + igain3*asum2, itime3 ; Second Inner Feedback

       zaw    aout, ioutch

       endin


;----------------------------------------------------------------------------------
; Output For reverb
       instr  15

idur   =      p3
igain  =      p4
iinch  =      p5

kdclik linseg 0, .002, igain, idur-.004, igain, .002, 0

ain    zar    iinch
       outs   ain*kdclik, -ain*kdclik  ; Inverting one side makes the sound
       endin                           ; seem to come from all around you.
                                       ; This may cause some problems with certain
                                       ; surround sound systems.
